#include <assert.h>
#include <string.h>
#include <stdint.h>
#include "basics.h"
#include "class.h"
#include "objects.h"

/* Auxiliary functions for working with objects and fields.
   These versions assume that no inheritance is allowed,
   and that all fields have size 1. */

/* Compute size of objects (in words) for given class. */
int count_object_slots(Class *class) {
  assert(STR_EQ(get_class(class,class->super_class),OBJECT));
  Field *f;
  int count = 0;
  for (f = class->fields; f < class->fields + class->fields_count; f++)
    if ((f->access_flags & ACC_STATIC) == 0) {
      assert (size_descriptor(get_utf8(class,f->descriptor_index)) == 1);
      count ++;
    } 
  return count;
}

/* Compute field offset in object. */
int object_field_offset(Class *class,char *name,char *desc) {
  assert(STR_EQ(get_class(class,class->super_class),OBJECT));
  Field *f;
  int offset = 0;
  for (f = class->fields; f < class->fields + class->fields_count; f++) 
    if ((f->access_flags & ACC_STATIC) == 0) {
      char *f_desc = get_utf8(class,f->descriptor_index);
      if (STR_EQ(name,get_utf8(class,f->name_index)) && STR_EQ(desc,f_desc)) /* type comparison is probably superfluous */
	return offset;
      else
	offset += size_descriptor(f_desc);
    }
  die("field %s not found in class %s",name,get_class(class,class->this_class));
}

/* Compute vector of object slot types for class. */
char **object_slot_types(Class *class) {
  assert(STR_EQ(get_class(class,class->super_class),OBJECT));
  Field *f;
  int count = count_object_slots(class);
  char **types = alloc(sizeof(char *) * count);
  int i = 0;
  for (f = class->fields; f < class->fields + class->fields_count; f++)
    if ((f->access_flags & ACC_STATIC) == 0) 
      types[i++] = get_utf8(class,f->descriptor_index);
  return types;
}

